@@ -1,7 +1,7 @@
 pfCompositeLF=function(TR,hw=250,
                        tarAge=NULL,binhw=NULL,
                        nboot=1000,conf=c(0.05,0.95),
-                       pseudodata=FALSE)
+                       pseudodata=FALSE,verbose=TRUE)
 {
   
   ## IF TR is a matrix
@@ -21,8 +21,7 @@ pfCompositeLF=function(TR,hw=250,
     colnames(TransData)=ID
     colnames(Age)=ID
     TR=structure(list(Age=structure(Age,class="matrix") ,TransData=structure
-                      (TransData,class="matrix"),Method="unspecified"))
-    
+                      (TransData,class="matrix"),Method="unspecified")) 
   }
   
   ## Prebinning procedure
@@ -43,37 +42,23 @@ pfCompositeLF=function(TR,hw=250,
   m=length(TR$TransData[,1])
   n=length(TR$TransData[1,])
   
-  #   #   ## Smmoth before prebinning
-  #   Stransdata=matrix(ncol=n,nrow=m)
-  #   for (i in 1:n){
-  #     tmp=cbind(TR$Age[,i],TR$TransData[,i])
-  #     tmp=na.omit(tmp)
-  #     x=as.vector(tmp[,1])
-  #     y=as.vector(tmp[,2])
-  #     locboot <- locfit(y ~ lp(x, deg = 1, h = hw), maxk = 500, family = "qrgauss")
-  #     predboot <- predict(locboot, newdata = x, se.fit = TRUE)
-  #     Stransdata[,i]=c(predboot$fit,rep(NA,m-length(predboot$fit)))    
-  #     #plot(TR$Age[,i],TR$TransData[,i])
-  #     #points(TR$Age[,i],Stransdata[,i],col="red")
-  #     #q=cbind(TR$Age[,i],Stransdata[,i])
-  #   }
-  
   # Matrix to store results
   result=matrix(ncol=length(TR$Age[1,]),nrow=length(tarAge))
-  #sm_result=matrix(ncol=length(IDn),nrow=length(tarAge)-1)
   
   ## Use non-overlapping bins by default if unspecified
   if(is.null(binhw))
     binhw=(tarAge[2]-tarAge[1])/2
   
   ## Prebinning procedure
-#   for (k in 1:n){
-#     c1 <- cut(TR$Age[,k], breaks = tarAge)
-#     tmean=tapply(TR$TransData[,k], c1, median)
-#     #tmean=tapply(Stransdata[,i], c1, mean)
-#     result[,k]=c(as.numeric(tmean))
-  #   } 
-  cat("Prebinning site: ")
+
+  if(verbose==TRUE){
+    percent=seq(10,100,by=10)
+    values=round(percent*n/100)
+    cat("Prebinning...")
+    cat("\n")
+    cat("Percentage done: ")
+  }
+  
   for (k in 1:n){
     if(length(TR$TransData[is.na(TR$TransData[,k])==FALSE,k])!=0){
       for (i in 1:length(tarAge)){
@@ -82,21 +67,31 @@ pfCompositeLF=function(TR,hw=250,
       }
     }
     ## print
-    cat("", k )
+    if(k %in% values & verbose==TRUE)
+      cat(percent[values==k]," ",sep="")
   }
-  cat("\n")
+  if(verbose==TRUE) 
+    cat("\n")
+  
   # suppres Inf values occuring with specific charcoal series (binary series)
   result[!is.finite(result)]=NA
   
   # Target Ages:
   centres=tarAge
-    
+  
   # Matrix to strore boot results
   mboot=matrix(nrow=length(centres),ncol=nboot)
   
-  #plot(NULL, xlab = "age", ylab = "locfit_500", ylim = c(-1, 1), xlim = c(0, 8000),type="n")
   set.seed(1)
-  cat("# of Bootstrap:")
+  
+  if(verbose==TRUE){
+    percent=seq(10,100,by=10)
+    values=round(percent*nboot/100)
+    cat("Bootstrapping...")
+    cat("\n")
+    cat("Percentage done: ")
+  }
+  
   ## Bootstrap procedure (with locfit)
   for (i in 1:nboot){
     ne=sample(seq(1,length(result[1,]),1),length(result[1,]),replace=TRUE)
@@ -130,11 +125,13 @@ pfCompositeLF=function(TR,hw=250,
     x=as.vector(dat$x)
     y=as.vector(dat$y)
     locboot <- locfit(y ~ lp(x, deg = 1, h = hw), maxk = 2000, family = "qrgauss")
+    if(is.na( locboot$dp[7])==FALSE){
     predboot <- predict(locboot, newdata = centres, se.fit = TRUE)
-    mboot[, i] <- predboot$fit
-    # note plotting lines is slowww
-    #lines(centres, mboot[, i], lwd = 2, col = rgb(0.5, 0.5, 0.5, 0.1))
-    if(i %in% seq(0,nboot,10)) cat("", i)
+    mboot[, i] <- predboot$fit}
+    
+    # Verbose
+    if(i %in% values & verbose==TRUE)
+      cat(percent[values==i]," ",sep="")
   }
   
   bootci=t(apply(mboot, 1, quantile, probs = conf,  na.rm = TRUE))
@@ -173,11 +170,6 @@ pfCompositeLF=function(TR,hw=250,
   predbootA <- predict(locbootA, newdata = centres, se.fit = TRUE)
   locfitAll <- predbootA$fit
   
-  
-  #lines(centres, bootci[,1], lwd = 2, col = "black")
-  #lines(centres, bootci[,3], lwd = 2, col ="black")
-  #lines(centres, bootmean, lwd = 2, col = "black")
-  
   # write out the transformed data
   result2=as.data.frame(cbind(centres,locfitAll,t(bootmean),bootci))
   colnames(result2)=c("AGE","LocFit","MEAN(of_boot)",as.character(conf))
@@ -198,6 +190,8 @@ pfCompositeLF=function(TR,hw=250,
                         BootCi=structure(bootci,row.names = as.character(centres),class = "matrix") ))
   class(output)="pfCompositeLF"
   return(output)  
+  if(verbose==TRUE) 
+    cat("\n")
 }
 
 ######SUMMARY########
@@ -205,10 +199,17 @@ pfCompositeLF=function(TR,hw=250,
 
 ######PLOT########
 
-plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="jet",...){
+plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="jet",xlim=NULL,
+                            ylim=NULL,main="Composite",text=FALSE,...){
   # Value for plotting:
   w=(x$BinCentres[2]-x$BinCentres[1])/2
   
+  # Lims
+  if (type=="ci") bootci1=t(apply(x$mboot, 1, quantile, probs = conf,  na.rm = TRUE)) 
+  if (type=="prctile") bootci1=t(apply(x$mboot, 1, quantile, probs = seq(0, 1, .01),  na.rm = TRUE))
+  if(is.null(xlim)) xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w)
+  if(is.null(ylim)) ylim= c(min(bootci1,na.rm=T),max(bootci1,na.rm=T))
+  
   if (type=="ci"){
     
     if(add=="sitenum")
@@ -216,8 +217,9 @@ plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="je
     
     bootci1=t(apply(x$mboot, 1, quantile, probs = conf,  na.rm = TRUE))    
     
-    plot(x$BinCentres,x$BootMean, xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w), ylim= c(min(bootci1,na.rm=T),max(bootci1,na.rm=T)), axes=F, mgp=c(2,0,0),
-         main=paste("Composite"), font.main=1, lab=c(8,5,5), 
+    plot(x$BinCentres,x$BootMean, 
+         xlim=xlim, ylim=ylim, axes=F, mgp=c(2,0,0),
+         main=main, font.main=1, lab=c(8,5,5), 
          ylab="Composite", xlab="Age (cal yr BP)", cex.lab=1, pch=16, cex=0.5, type="l")
     axis(1); axis(2, cex.axis=1)
     axis(side = 1, at = seq(0, 99000, by = 500), 
@@ -225,12 +227,13 @@ plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="je
     for (i in 1:length(conf)){
       lines(x$BinCentres,bootci1[,i],lty=2)
       pos=which.min(is.na(bootci1[,i]))
-      text(min(x$BinCentres)-200,bootci1[pos,i],paste(conf[i]*100,"%",sep=""),col="black")
+      if(text==TRUE) text(min(x$BinCentres)-200,bootci1[pos,i],paste(conf[i]*100,"%",sep=""),col="black")
     }
     # Plot site number
     if(add=="sitenum"){
       sitenum=length(x$BinnedData[1,])-rowSums(is.na(x$BinnedData))
-      plot(x$BinCentres,sitenum,xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w), ylim= c(min(sitenum,na.rm=T),max(sitenum,na.rm=T)), axes=F, mgp=c(2,0,0),
+      plot(x$BinCentres,sitenum,xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w), 
+           ylim= c(min(sitenum,na.rm=T),max(sitenum,na.rm=T)), axes=F, mgp=c(2,0,0),
            main=paste("Sites #"), font.main=1, lab=c(8,5,5), 
            ylab="Sites #", xlab="Age", cex.lab=0.8, pch=16, cex=0.5, type="o")
       axis(1); axis(2, cex.axis=1)
@@ -245,7 +248,9 @@ plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="je
     bootci1=bootci1[is.na(bootci1[,1])==FALSE,]
     n=length(bootci1[1,])
     ## PLOT
-    plot(NULL, type = "n", xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w), ylim = range(bootci1),axes=FALSE,ylab="Composite",xlab="Age",main="Percentiles")
+    plot(NULL, type = "n", 
+         xlim=xlim, 
+         ylim=ylim,axes=FALSE,ylab="Composite",xlab="Age",main=main)
     if(palette=="jet"){pal = colorRampPalette(rev(c("#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow", "#FF7F00", "red", "#7F0000")))} 
     if(palette=="BW"){
       pal = colorRampPalette(rev(c("white","black")))}
@@ -257,7 +262,7 @@ plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="je
     }
     for (i in c(2,11,51,91,100)){
       lines(bins1,bootci1[,i],col="grey",lty=2)
-      text(min(bins1)-200,median(bootci1[1:round(length(bootci1[,1])*0.02),i]),paste(i-1,"%",sep=""),col="grey")
+      if(text==TRUE) text(min(bins1)-200,median(bootci1[1:round(length(bootci1[,1])*0.02),i]),paste(i-1,"%",sep=""),col="grey")
     }
     axis(1)
     axis(side = 1, at = seq(0, 99000, by = 500), 
@@ -280,7 +285,8 @@ plot.pfCompositeLF=function(x,type="ci",add="NULL",conf=c(0.05,0.95),palette="je
     if(palette=="jet"){pal = colorRampPalette((c("#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow", "#FF7F00", "red", "#7F0000")))} 
     if(palette=="BW"){
       pal = colorRampPalette((c("white","black")))}
-    image(x$BinCentres, seqI, t(img),col = pal(100),xlab="Age",ylab="Composite",main="Density plot",axes=F, xlim=c(max(x$BinCentres)+w,min(x$BinCentres)-w))
+    image(x$BinCentres, seqI, t(img),col = pal(100),xlab="Age",ylab="Composite",main=main,axes=F,
+          xlim=xlim)
     axis(1, cex.axis=1, xaxp=c(0,99000,99)); axis(2, cex.axis=1)
     lines(x$BinCentres,rowMeans(x$mboot, na.rm=T))
     z=matrix(1:100,nrow=1)
